/************************************************************************************
DeviceControl RT-Component
Copyright (c) 2010, Kyoto Uni.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//=============================================================================//
/*!
 *	@file CSerial.h
 *	@brief Class for boost based Serial communication class 
 *  @author kazuyuki kon @ Kyoto-u
 */
//=============================================================================//
#ifndef __CSERIAL_H
#define __CSERIAL_H

#include <boost/asio.hpp>
#include <boost/bind.hpp>
#include <boost/thread.hpp>
#include <iostream>
#include <boost/circular_buffer.hpp>
#include <list>
#include <algorithm>

using namespace boost::asio;

const int BUFFERSIZE = 128;				//!< Maximum size of buffer
const int INTERMEDIATE_BUFFER = 20000;	//!< Maximum size of intermediate bufffer 

//=============================================================================//
/*!
 * @class CSerial
 */
//=============================================================================//
class CSerial
{

public:
	/*!
	 * @brief Default constructor(Do read/write process)
	 */
	CSerial();

	/*!
	 * @brief Destructor
	 */
	~CSerial();

	/*!
	 *	@brief Open serial port( do Sync)
	 *	@retval 
	 */
	bool openPort();

	/*!
	 * @brief Copy constructor
	 * @param[in] _doAsyncRead (true : do Async, false: Sync)
	 */
	bool openPort(bool _doAsyncRead);

	/*!
	 *	@brief close serial port
	 */
	void closePort();

	/*!
	 *  @brief set parity
	 *	@param[in] _parity set one of the enum PARITY member
	                       serial_port_base::parity::none
						   serial_port_base::parity::even
						   serial_port_base::parity::odd
	 */
	void setParity( int _parity )
	{
		if(_parity == serial_port_base::parity::none ||
           _parity == serial_port_base::parity::even ||
		   _parity == serial_port_base::parity::odd)
			m_parity = _parity;
	};

	/*!
	 *	@brief set device name
	 *	@param[in] _portName is a name of device (e.g. "COM2" or "/dev/ttyS0")
	 */
	void setPortName( std::string _portName )
	{
		m_portName = _portName;
	}

	/*!
	 *	@brief set baudrate
	 *	@param[in] _baudrate is a target baudrate
	 */
	void setBaudrate( int _baudrate )
	{
		if( _baudrate >0 && _baudrate <= 1000000)
		m_baudrate = _baudrate;
	}

	/*!
	 * @brief set stop bit
	 * @param[in] _stopBit is the size of stop bit (1 or 1.5 or 2)
						      serial_port_base::stop_bits::one
						      serial_port_base::stop_bits::two
	 */
	void setStopBit( int _stopBit )
	{
		if( _stopBit == serial_port_base::stop_bits::one          || 
			_stopBit == serial_port_base::stop_bits::onepointfive || 
			_stopBit == serial_port_base::stop_bits::two)
			m_stopBit = _stopBit;
	}

	/*!
	 * @brief set character size
	 * @param[in] _size is the size of chracter (7 or 8)
	 */
	void setCharacterSize( int _size )
	{
		if( _size == 7 || _size== 8 )
		{
			m_characterSize = _size;
		}
	}

	/*!
	 * @brief Set flow control
	 * @param[in] _flowControl is set based on enum FLOWCONTROL
	 */
	void setFlowControl( int _flowControl )
	{
		if( _flowControl == serial_port_base::flow_control::none     ||
			_flowControl == serial_port_base::flow_control::hardware ||
			_flowControl == serial_port_base::flow_control::software)
			m_flowControl = _flowControl;	
	}

	/*!
	 *	@brief Get connection status of serial communication
	 *	@retval is a connection status
	 */
    bool getConnection()
	{
		if( m_port ==NULL)return false;
		return m_port->is_open();
	};


	/*!
	*	@brief Get number of read data
	*	@retval Number of read data
	*/
    size_t getReadDataSize()
	{
		return m_readData.size();
	};

	/*!
	*	@brief Write one character 
	*/
	size_t write(const char _command);

	/*!
	*	@brief Write one character 
	*/
	size_t write(const void* _command, size_t _size);


	/*!
	*	@brief Read one character ( wait for receving )
	*   TODO : implement!!
	*/	
	size_t read(char* _command){};


	/*!
	*	@brief Read one character ( wait for receving )
	*/	
	size_t read(char* _command, size_t _size);


	/*!
	*	@brief Flush read buffer
	*/
	void flushReadBuffer()
	{
		{
			boost::mutex::scoped_lock(m_mutex);
			memset(m_rbuffer,0,BUFFERSIZE);
		}//mutex will be released 
	}

	/*!
	*	@brief Flush all buffers
	*/
	void flushAll()
	{
		{
			boost::mutex::scoped_lock(m_mutex);
			memset(m_rbuffer,0,BUFFERSIZE);
			m_readData.clear();
		}//mutex will be released 
	}

protected:

	/*!
	*	@brief Thread for (async) serial communication
	*	@note This function is called only if m_async is true 
	*/
	void run()
	{
		//std::cout<<"thread "<<inst<<" "<<count<<std::endl;
     	boost::system::error_code ignored;
		if( getConnection() )m_ioserv->run( ignored );
		//std::cout<<ignored.message()<<std::endl;
	};


	/*!
	*	@brief Callback function for asyncronously read
	*	@param[in] e 
	*	@param[in] _size
	*/
	void readCallback( const boost::system::error_code &e, size_t _size );

private:

	io_service *m_ioserv;		//!< IO service
    serial_port *m_port;		//!< Serial port
	bool doTerm;				//!< 

	std::string m_portName;		//!< Name of serial port
	int  m_baudrate; 			//!< Baudrate[bps]
	int  m_flowControl;  		//!< FlowControl setting
	int  m_characterSize;		//!< Character size 
	int  m_stopBit; 			//!< Stop bit
	int  m_parity;   			//!< Parity(NONE,EVEN,ODD) setting
	
	bool m_doAsyncRead;			//!< Read asynchronously, if m_doAsyncRead = true
	char m_rbuffer[BUFFERSIZE]; //!< Receive buffer
	std::list<char> m_readData;	//!< Read data

	boost::thread* m_thread;	//!< Thread handle
	boost::mutex m_mutex;		//!< Mutex for m_readData;

	int count;
	static int inst;

};


#endif
