/************************************************************************************
DeviceControl RT-Component
Copyright (c) 2010, Kyoto Uni.
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

// -*- C++ -*-
/*!
 * @file  DeviceControl.cpp
 * @brief RTC controlling an EMO button and a Patolight
 * @date $Date$
 *
 * $Id$
 */

#include "DeviceControl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* devicecontrol_spec[] =
  {
    "implementation_id", "DeviceControl",
    "type_name",         "DeviceControl",
    "description",       "RTC controlling an EMO button and a Patolight",
    "version",           "1.0.0",
    "vendor",            "MatsunoLab. @ Kyoto-u",
    "category",          "device",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
#ifdef _WIN32
    "conf.default.portName","COM16",
#else
    "conf.default.portName","/dev/ttyUSB0",
#endif
    // Widget
    "conf.__widget__.portName", "text",
    // Constraints
    "",
    "conf.default.baudrate","115200",
    ""

  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
DeviceControl::DeviceControl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_lightColorIn("lightColor", m_lightColor),
    m_emoStateOut("emoState", m_emoState)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
DeviceControl::~DeviceControl()
{
}



RTC::ReturnCode_t DeviceControl::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("lightColor", m_lightColorIn);
  
  // Set OutPort buffer
  addOutPort("emoState", m_emoStateOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
#ifdef _WIN32
   bindParameter("portName",m_portName,"COM16");
#else
   bindParameter("portName",m_portName,"/dev/ttyUSB0");
#endif
   bindParameter("baudrate",m_baudrate,"115200");
   m_configsets.update("default");  

   printf("---------------------------------------\n");
   printf("DeviceControl Module\n");
   printf("---------------------------------------\n");

  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DeviceControl::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DeviceControl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DeviceControl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t DeviceControl::onActivated(RTC::UniqueId ec_id)
{
#ifdef _WIN32
  std::string port = "\\\\.\\" + m_portName;
  std::cout<<"Connecting to " <<port<<std::endl;
#else
  std::string port = m_portName;
  std::cout<<"Connecting to " <<port<<std::endl;
#endif
  if( !dacs.initialize( port,m_baudrate ) )
  {
    return RTC::RTC_ERROR;
  }
  m_emoState.data = true;
  tv_start = coil::gettimeofday();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t DeviceControl::onDeactivated(RTC::UniqueId ec_id)
{
  dacs.setLightMode(0);
  dacs.update();
  coil::usleep(100000);

  dacs.terminate();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t DeviceControl::onExecute(RTC::UniqueId ec_id)
{
 //get emergency buttton status
  dacs.update();

  //Set color of patolight 
  if(m_lightColorIn.isNew() )
  {
	m_lightColorIn.read();
	switch( m_lightColor.data )
	{
		case 1:
			dacs.setLightMode(1);
			break;
		case 2:
			dacs.setLightMode(2);
			break;
		case 3:
			dacs.setLightMode(3);
			break;
		default:
			dacs.setLightMode(0);
			break;
	}
  }
  
  coil::TimeValue tv = coil::gettimeofday() - tv_start;

  //Get EMO status
  if( dacs.getEMOStatus() )
  {
	  printf("EMO Status : Emergency!!! @%lf \r",(double)tv.sec()+(double)tv.usec()/1000000);
	  m_emoState.tm.sec  = tv.sec();
	  m_emoState.tm.nsec = tv.usec()*1000;
      m_emoState.data = true;
  }
  else
  {
	  printf("EMO Status : normal       @%lf \r",(double)tv.sec()+(double)tv.usec()/1000000);
	  m_emoState.tm.sec  = tv.sec();
	  m_emoState.tm.nsec = tv.usec()*1000;
      m_emoState.data = false;
  }
  m_emoStateOut.write();
 
  return RTC::RTC_OK;
}


RTC::ReturnCode_t DeviceControl::onAborting(RTC::UniqueId ec_id)
{
  dacs.terminate();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t DeviceControl::onError(RTC::UniqueId ec_id)
{
  dacs.terminate();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t DeviceControl::onReset(RTC::UniqueId ec_id)
{
  dacs.terminate();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DeviceControl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DeviceControl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DeviceControlInit(RTC::Manager* manager)
  {
    coil::Properties profile(devicecontrol_spec);
    manager->registerFactory(profile,
                             RTC::Create<DeviceControl>,
                             RTC::Delete<DeviceControl>);
  }
  
};


